/*
*    This program demostrates calling a real mode procedure from a
*  protected mode program and passing it a real mode callback address.
*  The real mode procedure uses the callback address to call a protected
*  mode procedure. This has been tested under DOS, Windows and Warp.
*
*   1/24/95   JGD
*/




#include <dos.h>
#include <conio.h>
#include <iostream.h>
#include <windows.h>
#include <iomanip.h>

#include "call.h"



DPMIRegs regs_r;        // Used in call to int 31,301h
DPMIRegs regs_p;        // Used in call to int 31,303h

void AllocateRMC(unsigned int *, unsigned int *);


void main()
{
   unsigned int rmc_seg = 0, rmc_off = 0;

   clrscr();


   // First allocate real mode callback to pass to
   // realfunc (so it can call protfunc)
   AllocateRMC( &rmc_seg, &rmc_off );


   // Get the physical address of the real mode function (20bit)
   unsigned long realseg = GetSelectorBase( FP_SEG( realfunc ) ) ;

   // Segment of realfunc
   regs_r.cs = realseg >> 4;
   // Offset of realfunc
   regs_r.ip = realseg & 0x0f;

   // Segment of real mode callback
   regs_r.ax = rmc_seg;
   // Offset of real mode callback
   regs_r.dx = rmc_off;


   asm push ds               // Preserve the DS register
   _BX = 0;                  // zero out flags.
   _CX = 0;                  // Don't copy any words from prot stack to real stack
   _ES = FP_SEG( &regs_r );  // Selector of regs structure
   _DI = FP_OFF( &regs_r );  // Offset of regs structure
   asm pop ds                // Restore DS
   _AX = 0x301;              // Call Real mode function with far return frame

   geninterrupt( 0x31 );

   cout << "Made it back to protected mode." << endl;
}


// This function calls int 31,303h to allocate a real mode callback address
// for the function protfunc.
void AllocateRMC(unsigned int *seg, unsigned int* off)
{
   asm push ds                 // Preserve ES and DS
   asm push es

   _SI = FP_OFF( protfunc );   // Offset of prot. func.
   _ES = FP_SEG( &regs_p );    // Selector of regs struct for use in callback
   _DS = FP_SEG( protfunc );   // Selector of protected mode func.
   _DI = FP_OFF( &regs_p );    // Offset of regs struct
   _AX = 0x303;                // Allocate Real mode callback

   geninterrupt( 0x31 );

   *seg = _CX;
   *off = _DX;

   asm pop es                  // Restore ES and DS
   asm pop ds

}


