/*
*    Protected mode procedure used in callback. Note this is an interrupt
*  procedure. The DPMI spec states that the callback procedure must return
*  using an IRET.
*
*  When this function is called by via a realmode callback, we know
*  four things (via DPMI spec):
*     CS:IP = selector:offset specified in the original call to int 31,303h
*     DS:SI = selector:offset corresponding to real mode stack
*     ES:DI = selector:offset of real mode register data struct
*     SS:SP = locked protected mode stack
*/


#include <dos.h>
#include <iomanip.h>
#include <iostream.h>
#include "call.h"


void far interrupt protfunc()
{
   // We need access to the real mode stack in order to get the return
   // address, and we need to get to the real mode register data in order
   // to set the address.

   unsigned int ret_seg, ret_off;

   // The interrupt keyword causes the values of all registers to be pushed
   // on the stack. If you look at the generated assembly, you will see that
   // DS is then set to the value of BP. We need the old value so I retieve it
   // from the stack.

   asm {
       mov ax, [bp+6]       // Retrieve old value from the stack
       mov ds, ax
       mov ax, ds:[si+2]    // Obtain segment of return address from
       mov ret_seg, ax      // the stack of the real mode procedure
       mov ax, ds:[si]      // Obtain offset of return address
       mov ret_off, ax
   }

   // Create pointer to real mode register struct.
   DPMIRegs * real_regs = (DPMIRegs*) MK_FP( _ES, _DI );

   // Blast to video memory to show we got here (in GREEN).
   unsigned int * p = (unsigned int *) MK_FP( __SegB800, 21*160 );
   for( int i = 0; i < 26 ; i++ )
         p[i] = 0x0200 + ( 'a' + i );



   real_regs->rsvd = 0;          // Make sure reserved field is zero
   real_regs->sp += 4;           // Clean up stack
   real_regs->cs = ret_seg;      // Set return address.
   real_regs->ip = ret_off;

}
